from calendar import prmonth
import os
import zipfile
import numpy as np
import pandas as pd
from pathlib import Path
import xml.etree.ElementTree as xmltree
from xml.dom import minidom
import click


class Config:
    name_columns = ['Price',  'Includes Bills']
    description_columns = ['Url', 'Min Stay', 'Notes']
    location_column = 'Location'

    @staticmethod
    def set_name_columns(name_columns):
        if isinstance(name_columns, str):
            name_columns = name_columns.split(',')
        Config.name_columns = name_columns

    @staticmethod
    def set_description_columns(description_columns):
        if isinstance(description_columns, str):
            description_columns = description_columns.split(',')
        Config.description_columns = description_columns

    @staticmethod
    def set_location_column(location_column):
        assert isinstance(location_column, str)
        Config.location_column = location_column


def load_kmz(kmz_file_path):
    print(Path(kmz_file_path).resolve())
    print(Path(kmz_file_path).resolve().exists())
    file_to_extract = 'doc.kml'
    xml = None
    try:
        with zipfile.ZipFile(kmz_file_path) as z:
            xml = xmltree.fromstring(z.read(file_to_extract))
    except:
        print("Invalid file")
    return xml


def load_xlsx(xlsx_file_path):
    df = pd.read_excel(xlsx_file_path)
    return df.to_dict('records')


def remove_all_current_placemarkers(xml):
    doc_elements = xml.find('{http://www.opengis.net/kml/2.2}Document')
    for stuff in doc_elements.findall('{http://www.opengis.net/kml/2.2}Placemark'):
        doc_elements.remove(stuff)
    return xml


def append_placemarks_to_xml(xml, placemarks):
    doc = xml.find('{http://www.opengis.net/kml/2.2}Document')
    for placemark in placemarks:
        doc.append(placemark)
    return xml


def map_dit(dict_list):
    items = list()

    for item in dict_list:
        name = ' '.join([str(item[x]) for x in Config.name_columns])
        description = '\n'.join([str(item[x]) for x in Config.description_columns if item[x] is not np.nan])
        location = parse_location(item[Config.location_column])
        items.append(dict(name=name, description=description, location=location))
    return items


def parse_location(text_location):
    location_coordinates = text_location.replace(' ', '').split(',')
    location_coordinates = [location_coordinates[1] , location_coordinates[0]]
    location_coordinates.append('0')
    location = ','.join(location_coordinates)
    return location


def map_dict_element_to_placemark_xml(element: dict):
    xml_element = xmltree.Element('{http://www.opengis.net/kml/2.2}Placemark')
    xml_name = xmltree.SubElement(xml_element, '{http://www.opengis.net/kml/2.2}name')
    xml_name.text = element['name']
    xml_description = xmltree.SubElement(xml_element, '{http://www.opengis.net/kml/2.2}description')
    xml_description.text = element['description']
    xml_style_url = xmltree.SubElement(xml_element, '{http://www.opengis.net/kml/2.2}styleUrl')
    xml_style_url.text = '#icon-1603-E65100'
    xml_point = xmltree.SubElement(xml_element, '{http://www.opengis.net/kml/2.2}Point')
    xml_coordinates = xmltree.SubElement(xml_point, '{http://www.opengis.net/kml/2.2}coordinates')
    xml_coordinates.text = element['location']
    return xml_element


def process(xlsx_path, base_kmz_path, output_path):
    dict_elements = map_dit(load_xlsx(xlsx_path))
    placemarks = [map_dict_element_to_placemark_xml(dict_element) for dict_element in dict_elements]

    kmz_xml = load_kmz(base_kmz_path)
    kmz_xml = remove_all_current_placemarkers(kmz_xml)
    kmz_xml = append_placemarks_to_xml(kmz_xml, placemarks)

    xmltree.register_namespace("", 'http://www.opengis.net/kml/2.2') 
    tree = xmltree.ElementTree(kmz_xml)

    with open (output_path, "w") as file:
        rough_string = xmltree.tostring(kmz_xml, 'utf-8')
        reparsed = minidom.parseString(rough_string)
        content = reparsed.toprettyxml(indent="\t")
        file.write(content)

        
import click

@click.command()
@click.option("-i", "--xlsx-path", prompt="XLSX containign data to convert to kml.")
@click.option("-o", "--output-path", default='output.kml', prompt="Output fiel path.", help="Where to save output.")
@click.option("-b", "--kmz-path", default='data/default.kmz', help="Base kmz file that provides templating.")
@click.option("-n", "--name", default='name', prompt='Columns to be used as name.')
@click.option("-d", "--description", default='description', prompt='Columns to be used as description.')
@click.option("-l", "--location", default='locaiton', prompt='Columns to be used as location.')
def convert_xlsx_to_kml(xlsx_path, kmz_path, output_path, name, description, location):
    Config.set_name_columns(name)
    Config.set_description_columns(description)
    Config.set_location_column(location)

    process(
        xlsx_path, kmz_path, output_path
    )


if __name__ == '__main__':
    convert_xlsx_to_kml()
